@@ -161,7 +161,6 @@ static int pgp_send_menu (int bits, int *redraw)
     else
     {
       bits &= ~PGPSIGN;
-      mutt_error _("An unkown PGP version was defined for signing.");
     }
 
     *redraw = REDRAW_FULL;
@@ -650,6 +649,9 @@ int mutt_compose_menu (HEADER *msg,   /* structure for new message */
 	    break;
 
 	  mutt_expand_path (fname, sizeof (fname));
+#ifdef USE_IMAP
+          if (!mx_is_imap (fname))
+#endif
 	  /* check to make sure the file exists and is readable */
 	  if (access (fname, R_OK) == -1)
 	  {
@@ -890,7 +892,10 @@ int mutt_compose_menu (HEADER *msg,   /* structure for new message */
 	    b->hdr->content = NULL;
 	    mutt_free_header (&b->hdr);
 	  }
-	  
+
+	  /* this may have been a change to text/something */
+	  mutt_update_encoding (b);
+
 	  menu->redraw = REDRAW_CURRENT;
 	}
 	break;
