@@ -45,6 +45,7 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
 	       const char *prefix)
 {
   int from = 0;
+  int this_is_from;
   int ignore = 0;
   char buf[STRING]; /* should be long enough to get most fields in one pass */
   char *nl;
@@ -136,16 +137,19 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
     if (nl && buf[0] != ' ' && buf[0] != '\t')
     {
       ignore = 1;
+      this_is_from = 0;
       if (!from && mutt_strncmp ("From ", buf, 5) == 0)
       {
 	if ((flags & CH_FROM) == 0)
 	  continue;
-	from = 1;
+	this_is_from = from = 1;
       }
       else if (buf[0] == '\n' || (buf[0] == '\r' && buf[1] == '\n'))
 	break; /* end of header */
 
-      if ((flags & CH_WEED) && 
+      /* note: CH_FROM takes precedence over header weeding. */
+      if (!((flags & CH_FROM) && (flags & CH_FORCE_FROM) && this_is_from) &&
+	  (flags & CH_WEED) &&
 	  mutt_matches_ignore (buf, Ignore) &&
 	  !mutt_matches_ignore (buf, UnIgnore))
 	continue;
@@ -203,7 +207,7 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
     if (headers[x])
     {
       if (flags & CH_DECODE)
-	rfc2047_decode (headers[x], headers[x], mutt_strlen (headers[x]));
+	rfc2047_decode (headers[x], headers[x], mutt_strlen (headers[x]) + 1);
 
       /* We couldn't do the prefixing when reading because RFC 2047
        * decoding may have concatenated lines.
@@ -262,6 +266,7 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
 /* flags
  	CH_DECODE	RFC2047 header decoding
  	CH_FROM		retain the "From " message separator
+        CH_FORCE_FROM	give CH_FROM precedence over CH_WEED
  	CH_MIME		ignore MIME fields
 	CH_NOLEN	don't write Content-Length: and Lines:
  	CH_NONEWLINE	don't output a newline after the header
@@ -417,6 +422,7 @@ _mutt_copy_message (FILE *fpout, FILE *fpin, HEADER *hdr, BODY *body,
 {
   char prefix[SHORT_STRING];
   STATE s;
+  long new_offset = -1;
 
   if (flags & M_CM_PREFIX)
     _mutt_make_string (prefix, sizeof (prefix), NONULL (Prefix), Context, hdr, 0);
@@ -429,7 +435,6 @@ _mutt_copy_message (FILE *fpout, FILE *fpin, HEADER *hdr, BODY *body,
     else if (hdr->attach_del && (chflags & CH_UPDATE_LEN))
     {
       int new_lines;
-      long new_offset;
       long new_length = body->length;
       char date[SHORT_STRING];
 
@@ -488,6 +493,8 @@ _mutt_copy_message (FILE *fpout, FILE *fpin, HEADER *hdr, BODY *body,
     if (mutt_copy_header (fpin, hdr, fpout, chflags,
 			  (chflags & CH_PREFIX) ? prefix : NULL) == -1)
       return -1;
+
+    new_offset = ftell (fpout);
   }
 
   if (flags & M_CM_DECODE)
@@ -555,9 +562,17 @@ _mutt_copy_message (FILE *fpout, FILE *fpin, HEADER *hdr, BODY *body,
 	}
       } 
     }
-    else
-      return mutt_copy_bytes (fpin, fpout, body->length);
+    else if (mutt_copy_bytes (fpin, fpout, body->length) == -1)
+      return -1;
   }
+
+  if ((flags & M_CM_UPDATE) && (flags & M_CM_NOHEADER) == 0 
+      && new_offset != -1)
+  {
+    body->offset = new_offset;
+    mutt_free_body (&body->parts);
+  }
+
   return 0;
 }
 
@@ -596,11 +611,13 @@ _mutt_append_message (CONTEXT *dest, FILE *fpin, CONTEXT *src, HEADER *hdr,
   if ((msg = mx_open_new_message (dest, hdr, (src->magic == M_MBOX || src->magic == M_MMDF || src->magic == M_KENDRA) ? 0 : M_ADD_FROM)) == NULL)
     return -1;
   if (dest->magic == M_MBOX || dest->magic == M_MMDF || dest->magic == M_KENDRA)
-    chflags |= CH_FROM;
+    chflags |= CH_FROM | CH_FORCE_FROM;
   chflags |= (dest->magic == M_MAILDIR ? CH_NOSTATUS : CH_UPDATE);
   r = _mutt_copy_message (msg->fp, fpin, hdr, body, flags, chflags);
-  if (mx_close_message (&msg) != 0)
-    return (-1);
+  if (mx_commit_message (msg, dest) != 0)
+    r = -1;
+
+  mx_close_message (&msg);
   return r;
 }
 
