@@ -26,6 +26,8 @@ WHERE char Errorbuf[SHORT_STRING];
 WHERE char *MuttDotlock;
 #endif
 
+WHERE ADDRESS *From;
+
 WHERE char *AliasFile;
 WHERE char *AliasFmt;
 WHERE char *AttachSep;
@@ -50,7 +52,6 @@ WHERE char *ImapPass INITVAL (NULL);
 WHERE short ImapCheckTime;
 WHERE char *ImapHomeNamespace INITVAL (NULL);
 #endif
-WHERE char *InReplyTo;
 WHERE char *Inbox;
 WHERE char *Ispell;
 WHERE char *Locale;
@@ -100,6 +101,7 @@ WHERE LIST *HeaderOrderList INITVAL(0);
 WHERE LIST *Ignore INITVAL(0);
 WHERE LIST *UnIgnore INITVAL(0);
 WHERE LIST *MailLists INITVAL(0);
+WHERE LIST *SubscribedLists INITVAL(0);
 
 /* bit vector for boolean variables */
 #ifdef MAIN_C
@@ -109,7 +111,11 @@ extern unsigned char Options[];
 #endif
 
 /* bit vector for the yes/no/ask variable type */
-WHERE unsigned long QuadOptions INITVAL (0);
+#ifdef MAIN_C
+unsigned char QuadOptions[(OPT_MAX*2 + 7) / 8];
+#else
+extern unsigned char QuadOptions[];
+#endif
 
 WHERE unsigned short Counter INITVAL (0);
 
