@@ -26,10 +26,15 @@ int imap_check_mailbox (CONTEXT *ctx, int *index_hint);
 int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno);
 int imap_open_mailbox (CONTEXT *ctx);
 int imap_open_mailbox_append (CONTEXT *ctx);
-int imap_sync_mailbox (CONTEXT *ctx);
+int imap_select_mailbox (CONTEXT *ctx, const char* path);
+int imap_sync_mailbox (CONTEXT *ctx, int expunge);
 void imap_fastclose_mailbox (CONTEXT *ctx);
 int imap_buffy_check (char *path);
+int imap_mailbox_check (char *path, int new);
 int imap_subscribe (char *path, int subscribe);
 int imap_init_browse (char *path, struct browser_state *state);
+int imap_complete (char* dest, size_t dlen, char* path);
+void imap_qualify_path (char* dest, size_t len, const char* host, int port,
+  const char* path, const char* name);
 
 #endif
