@@ -43,17 +43,29 @@
 #include <errno.h>
 #include <sys/wait.h>
 
-#define toggle_quadoption(opt) QuadOptions ^= (1 << (2 * opt))
+void toggle_quadoption (int opt)
+{
+  int n = opt/4;
+  int b = (opt % 4) * 2;
+
+  QuadOptions[n] ^= (1 << b);
+}
 
 void set_quadoption (int opt, int flag)
 {
-  QuadOptions &= ~(0x3 << (2 * opt)); /* first clear the bits */
-  QuadOptions |= (flag & 0x3) << (2 * opt); /* now set them */
+  int n = opt/4;
+  int b = (opt % 4) * 2;
+
+  QuadOptions[n] &= ~(0x3 << b);
+  QuadOptions[n] |= (flag & 0x3) << b;
 }
 
 int quadoption (int opt)
 {
-  return ((QuadOptions >> (opt * 2)) & 0x3);
+  int n = opt/4;
+  int b = (opt % 4) * 2;
+
+  return (QuadOptions[n] >> b) & 0x3;
 }
 
 int query_quadoption (int opt, const char *prompt)
@@ -286,112 +298,144 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
   return 0;
 }
 
-void mutt_add_to_list (LIST **list, BUFFER *inp)
+static void add_to_list (LIST **list, const char *str)
 {
   LIST *t, *last = NULL;
-  BUFFER buf;
 
-  memset (&buf, 0, sizeof (buf));
-  do
+  /* check to make sure the item is not already on this list */
+  for (last = *list; last; last = last->next)
   {
-    mutt_extract_token (&buf, inp, 0);
-
-    /* check to make sure the item is not already on this list */
-    for (last = *list; last; last = last->next)
+    if (mutt_strcasecmp (str, last->data) == 0)
     {
-      if (mutt_strcasecmp (buf.data, last->data) == 0)
-      {
-	/* already on the list, so just ignore it */
-	last = NULL;
-	break;
-      }
-      if (!last->next)
-	break;
-    }
-
-    if (!*list || last)
-    {
-      t = (LIST *) safe_calloc (1, sizeof (LIST));
-      t->data = buf.data;
-      memset (&buf, 0, sizeof (buf));
-      if (last)
-      {
-	last->next = t;
-	last = last->next;
-      }
-      else
-	*list = last = t;
+      /* already on the list, so just ignore it */
+      last = NULL;
+      break;
     }
+    if (!last->next)
+      break;
+  }
+
+  if (!*list || last)
+  {
+    t = (LIST *) safe_calloc (1, sizeof (LIST));
+    t->data = safe_strdup (str);
+    if (last)
+    {
+      last->next = t;
+      last = last->next;
+    }
+    else
+      *list = last = t;
   }
-  while (MoreArgs (inp));
-  FREE (&buf.data);
 }
 
-static void remove_from_list (LIST **l, BUFFER *inp)
+static void remove_from_list (LIST **l, const char *str)
 {
   LIST *p, *last = NULL;
-  BUFFER buf;
 
-  memset (&buf, 0, sizeof (buf));
-  do
+  if (mutt_strcmp ("*", str) == 0)
+    mutt_free_list (l);    /* ``unCMD *'' means delete all current entries */
+  else
   {
-    mutt_extract_token (&buf, inp, 0);
-
-    if (mutt_strcmp ("*", buf.data) == 0)
-      mutt_free_list (l);    /* ``unCMD *'' means delete all current entries */
-    else
+    p = *l;
+    last = NULL;
+    while (p)
     {
-      p = *l;
-      last = NULL;
-      while (p)
+      if (mutt_strcasecmp (str, p->data) == 0)
       {
-	if (mutt_strcasecmp (buf.data, p->data) == 0)
-	{
-	  safe_free ((void **) &p->data);
-	  if (last)
-	    last->next = p->next;
-	  else
-	    (*l) = p->next;
-	  safe_free ((void **) &p);
-	}
+	safe_free ((void **) &p->data);
+	if (last)
+	  last->next = p->next;
 	else
-	{
-	  last = p;
-	  p = p->next;
-	}
+	  (*l) = p->next;
+	safe_free ((void **) &p);
+      }
+      else
+      {
+	last = p;
+	p = p->next;
       }
     }
   }
-  while (MoreArgs (inp));
-  FREE (&buf.data);
 }
 
 static int parse_unignore (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  mutt_add_to_list (&UnIgnore, s);
-  remove_from_list (&Ignore, s);
+  do
+  {
+    mutt_extract_token (buf, s, 0);
+    add_to_list (&UnIgnore, buf->data);
+    remove_from_list (&Ignore, buf->data);
+  }
+  while (MoreArgs (s));
+
   return 0;
 }
 
 static int parse_ignore (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  mutt_add_to_list (&Ignore, s);
-  remove_from_list (&UnIgnore, s);
+  do
+  {
+    mutt_extract_token (buf, s, 0);
+    remove_from_list (&UnIgnore, buf->data);
+    add_to_list (&Ignore, buf->data);
+  }
+  while (MoreArgs (s));
+
   return 0;
 }
 
 static int parse_list (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  mutt_add_to_list ((LIST **) data, s);
+  do
+  {
+    mutt_extract_token (buf, s, 0);
+    add_to_list ((LIST **) data, buf->data);
+  }
+  while (MoreArgs (s));
+
   return 0;
 }
 
 static int parse_unlist (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  remove_from_list ((LIST **) data, s);
+  do
+  {
+    mutt_extract_token (buf, s, 0);
+    remove_from_list ((LIST **) data, buf->data);
+  }
+  while (MoreArgs (s));
+
   return 0;
 }
 
+
+static int parse_unlists (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
+{
+  do
+  {
+    mutt_extract_token (buf, s, 0);
+    remove_from_list (&MailLists, buf->data);
+    remove_from_list (&SubscribedLists, buf->data);
+  }
+  while (MoreArgs (s));
+
+  return 0;
+}
+
+static int parse_subscribe (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
+{
+  do
+  {
+    mutt_extract_token (buf, s, 0);
+    add_to_list (&MailLists, buf->data);
+    add_to_list (&SubscribedLists, buf->data);
+  }
+  while (MoreArgs (s));
+
+  return 0;
+}
+  
 static int parse_unalias (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
   ALIAS *tmp, *last = NULL;
@@ -614,6 +658,13 @@ static void mutt_restore_default (struct option_t *p)
 	*((char **) p->data) = safe_strdup (path);
       }
       break;
+    case DT_ADDR:
+      if (p->init)
+      {
+	rfc822_free_address ((ADDRESS **) p->data);
+	*((ADDRESS **) p->data) = rfc822_parse_adrlist (NULL, (char *) p->init);
+      }
+      break;
     case DT_BOOL:
       if (p->init)
 	set_option (p->data);
@@ -782,31 +833,52 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 	set_option (MuttVars[idx].data);
     }
     else if (DTYPE (MuttVars[idx].type) == DT_STR ||
-	     DTYPE (MuttVars[idx].type) == DT_PATH)
+	     DTYPE (MuttVars[idx].type) == DT_PATH ||
+	     DTYPE (MuttVars[idx].type) == DT_ADDR)
     {
       if (query || *s->dptr != '=')
       {
+	char _tmp[STRING];
+	char *val = NULL;
+	
+	if (DTYPE (MuttVars[idx].type) == DT_ADDR)
+	{
+	  _tmp[0] = '\0';
+	  rfc822_write_address (_tmp, sizeof (_tmp), *((ADDRESS **) MuttVars[idx].data));
+	  val = _tmp;
+	}
+	else
+	  val = *((char **) MuttVars[idx].data);
+	
 	/* user requested the value of this variable */
 	snprintf (err->data, err->dsize, "%s=\"%s\"", MuttVars[idx].option,
-		  NONULL (*((char **) MuttVars[idx].data)));
+		  NONULL (val));
 	break;
       }
 
       s->dptr++;
 
       /* copy the value of the string */
-      FREE (MuttVars[idx].data);
+      if (DTYPE (MuttVars[idx].type) == DT_ADDR)
+	rfc822_free_address ((ADDRESS **) MuttVars[idx].data);
+      else
+	FREE (MuttVars[idx].data);
+
       mutt_extract_token (tmp, s, 0);
-      if (MuttVars[idx].type == DT_PATH)
+      if (DTYPE (MuttVars[idx].type) == DT_PATH)
       {
 	strfcpy (scratch, tmp->data, sizeof (scratch));
 	mutt_expand_path (scratch, sizeof (scratch));
 	*((char **) MuttVars[idx].data) = safe_strdup (scratch);
       }
-      else
+      else if (DTYPE (MuttVars[idx].type) == DT_STR)
       {
 	*((char **) MuttVars[idx].data) = safe_strdup (tmp->data);
       }
+      else
+      {
+	*((ADDRESS **) MuttVars[idx].data) = rfc822_parse_adrlist (NULL, tmp->data);
+      }
     }
     else if (DTYPE(MuttVars[idx].type) == DT_RX)
     {
@@ -1400,20 +1472,29 @@ int mutt_var_value_complete (char *buffer, size_t len, int pos)
       return 0; /* no such variable. */
     else
     {
-      char tmp [LONG_STRING];
+      char tmp [LONG_STRING], tmp2[LONG_STRING];
+      char *s, *d;
       size_t dlen = buffer + len - pt - spaces;
       char *vals[] = { "no", "yes", "ask-no", "ask-yes" };
-      strfcpy (tmp, pt, sizeof(tmp));
-      
+
+      tmp[0] = '\0';
+
       if ((DTYPE(MuttVars[idx].type) == DT_STR) || 
 	  (DTYPE(MuttVars[idx].type) == DT_PATH) ||
 	  (DTYPE(MuttVars[idx].type) == DT_RX))
-	snprintf(pt, dlen, "%s\"%s\"", tmp, 
-		  NONULL (*((char **) MuttVars[idx].data)));
+      {
+	strfcpy (tmp, NONULL (*((char **) MuttVars[idx].data)), sizeof (tmp));
+	if (DTYPE (MuttVars[idx].type) == DT_PATH)
+	  mutt_pretty_mailbox (tmp);
+      }
+      else if (DTYPE (MuttVars[idx].type) == DT_ADDR)
+      {
+	rfc822_write_address (tmp, sizeof (tmp), *((ADDRESS **) MuttVars[idx].data));
+      }
       else if (DTYPE (MuttVars[idx].type) == DT_QUAD)
-	snprintf(pt, dlen, "%s%s", tmp,  vals[quadoption (MuttVars[idx].data)]);
+	strfcpy (tmp, vals[quadoption (MuttVars[idx].data)], sizeof (tmp));
       else if (DTYPE (MuttVars[idx].type) == DT_NUM)
-	snprintf (pt, dlen, "%s%d", tmp, (*((short *) MuttVars[idx].data)));
+	snprintf (tmp, sizeof (tmp), "%d", (*((short *) MuttVars[idx].data)));
       else if (DTYPE (MuttVars[idx].type) == DT_SORT)
       {
 	const struct mapping_t *map;
@@ -1437,16 +1518,27 @@ int mutt_var_value_complete (char *buffer, size_t len, int pos)
 	    break;
 	}
 	p = mutt_getnamebyvalue (*((short *) MuttVars[idx].data) & SORT_MASK, map);
-	snprintf (pt, dlen, "%s\"%s%s%s\"", tmp,
+	snprintf (tmp, sizeof (tmp), "%s%s%s",
 		  (*((short *) MuttVars[idx].data) & SORT_REVERSE) ? "reverse-" : "",
 		  (*((short *) MuttVars[idx].data) & SORT_LAST) ? "last-" : "",
 		  p);
       }
       else if (DTYPE (MuttVars[idx].type) == DT_BOOL)
-	snprintf (pt, dlen, "%s%s", tmp, 
-		  option (MuttVars[idx].data) ? "yes" : "no");
+	strfcpy (tmp, option (MuttVars[idx].data) ? "yes" : "no", sizeof (tmp));
       else
 	return 0;
+      
+      for (s = tmp, d = tmp2; *s && (d - tmp2) < sizeof (tmp2) - 2;)
+      {
+	if (*s == '\\' || *s == '"')
+	  *d++ = '\\';
+	*d++ = *s++;
+      }
+      *d = '\0';
+      
+      strfcpy (tmp, pt, sizeof (tmp));
+      snprintf (pt, dlen, "%s\"%s\"", tmp, tmp2);
+	  
       return 1;
     }
   }
@@ -1734,5 +1826,7 @@ void mutt_init (int skip_sys_rc, LIST *commands)
       mutt_exit(1);
   }
 
+#if 0
   set_option (OPTWEED); /* turn weeding on by default */
+#endif
 }
