@@ -51,6 +51,7 @@ Copyright (C) 1996-9 Michael R. Elkins <me@cs.hmc.edu>\n\
 Copyright (C) 1996-9 Brandon Long <blong@fiction.net>\n\
 Copyright (C) 1997-9 Thomas Roessler <roessler@guug.de>\n\
 Copyright (C) 1998-9 Werner Koch <wk@isil.d.shuttle.de>\n\
+Copyright (C) 1999   Brendan Cully <brendan@kublai.com>\n\
 \n\
 Lots of others not mentioned here contributed lots of code,\n\
 fixes, and suggestions.\n\
@@ -223,6 +224,7 @@ static void show_version (void)
 
 
 #ifdef _PGPPATH
+	"\n"
 #ifdef HAVE_PGP5
 	"+HAVE_PGP5  "
 #endif
@@ -232,6 +234,7 @@ static void show_version (void)
 #ifdef HAVE_GPG
 	"+HAVE_GPG  "
 #endif
+	"\n"
 #endif
 
 
@@ -278,7 +281,7 @@ static void show_version (void)
 # ifdef _PGPV3PATH
   printf ("_PGPV3PATH=\"%s\"\n", _PGPV3PATH);
 # endif
-# ifdef _PGPGPPATH
+# ifdef _PGPGPGPATH
   printf ("_PGPGPGPATH=\"%s\"\n", _PGPGPGPATH);
 # endif
 #endif
@@ -369,7 +372,8 @@ int main (int argc, char **argv)
   umask (077);
 
   memset (Options, 0, sizeof (Options));
-
+  memset (QuadOptions, 0, sizeof (QuadOptions));
+  
   while ((i = getopt (argc, argv, "a:b:F:f:c:d:e:H:s:i:hm:npRvxyzZ")) != EOF)
     switch (i)
     {
@@ -508,6 +512,26 @@ int main (int argc, char **argv)
     mutt_error = mutt_curses_error;
   }
 
+  /* Create the ~/Mail directory if it doesn't exist. */
+  if (!option (OPTNOCURSES) && Maildir)
+  {
+    struct stat sb;
+    char fpath[_POSIX_PATH_MAX];
+    char msg[STRING];
+
+    strfcpy (fpath, Maildir, sizeof (fpath));
+    mutt_expand_path (fpath, sizeof (fpath));
+    if (stat (fpath, &sb) == -1 && errno == ENOENT)
+    {
+      snprintf (msg, sizeof (msg), _("%s does not exist. Create it?"), Maildir);
+      if (mutt_yesorno (msg, 1) == 1)
+      {
+	if (mkdir (fpath, 0700) == -1 && errno != EEXIST)
+	  mutt_error ( _("Can't create %s: %s."), Maildir, strerror (errno));
+      }
+    }
+  }
+
   if (sendflags & SENDPOSTPONED)
   {
     if (!option (OPTNOCURSES))
