@@ -140,7 +140,7 @@ void pgp_application_pgp_handler (BODY *m, STATE *s)
     offset = ftell (s->fpin);
     bytes -= (offset - last_pos); /* don't rely on mutt_strlen(buf) */
     last_pos = offset;
-
+    
     if (mutt_strncmp ("-----BEGIN PGP ", buf, 15) == 0)
     {
       clearsign = 0;
@@ -414,9 +414,6 @@ int mutt_is_application_pgp (BODY *m)
       if ((p = mutt_get_parameter ("x-action", m->parameter))
 	  && (!mutt_strcasecmp (p, "sign") || !mutt_strcasecmp (p, "signclear")))
 	t |= PGPSIGN;
-      else if((p = mutt_get_parameter ("format", m->parameter))
-	      && !mutt_strcasecmp(p, "keys-only"))
-	t |= PGPKEY;
 
       if ((p = mutt_get_parameter ("format", m->parameter)) && 
 	  !mutt_strcasecmp (p, "keys-only"))
@@ -443,20 +440,24 @@ int pgp_query (BODY *m)
   /* Check for PGP/MIME messages. */
   if (m->type == TYPEMULTIPART)
   {
-    if(mutt_is_multipart_signed(m))
+    if (mutt_is_multipart_signed(m))
       t |= PGPSIGN;
     else if (mutt_is_multipart_encrypted(m))
       t |= PGPENCRYPT;
+
+    if ((mutt_is_multipart_signed (m) || mutt_is_multipart_encrypted (m)) 
+	&& m->goodsig)
+      t |= PGPGOODSIGN;
   }
 
-  if(m->type == TYPEMULTIPART || m->type == TYPEMESSAGE)
+  if (m->type == TYPEMULTIPART || m->type == TYPEMESSAGE)
   {
     BODY *p;
  
-    for(p = m->parts; p; p = p->next)
-      t |= pgp_query(p);
+    for (p = m->parts; p; p = p->next)
+      t |= pgp_query(p) & ~PGPGOODSIGN;
   }
-  
+
   return t;
 }
 
@@ -522,6 +523,7 @@ static int pgp_verify_one (BODY *sigbdy, STATE *s, const char *tempfile)
   char sigfile[_POSIX_PATH_MAX], pgperrfile[_POSIX_PATH_MAX];
   FILE *fp, *pgpout, *pgperr;
   pid_t thepid;
+  int rv = -1;
   
   snprintf (sigfile, sizeof (sigfile), "%s.asc", tempfile);
   
@@ -556,7 +558,7 @@ static int pgp_verify_one (BODY *sigbdy, STATE *s, const char *tempfile)
     mutt_copy_stream(pgperr, s->fpout);
     fclose(pgperr);
     
-    mutt_wait_filter (thepid);
+    rv = mutt_wait_filter (thepid);
   }
   
   state_puts (_("[-- End of PGP output --]\n\n"), s);
@@ -564,7 +566,7 @@ static int pgp_verify_one (BODY *sigbdy, STATE *s, const char *tempfile)
   mutt_unlink (sigfile);
   mutt_unlink (pgperrfile);
 
-  return 0;
+  return rv;
 }
 
 /*
@@ -577,9 +579,11 @@ void pgp_signed_handler (BODY *a, STATE *s)
   int protocol_major = TYPEOTHER;
   char *protocol_minor = NULL;
   
+  BODY *b = a;
   BODY **signatures = NULL;
   int sigcnt = 0;
   int i;
+  short goodsig = 1;
 
   protocol = mutt_get_parameter ("protocol", a->parameter);
   a = a->parts;
@@ -632,7 +636,10 @@ void pgp_signed_handler (BODY *a, STATE *s)
 	{
 	  if (signatures[i]->type == TYPEAPPLICATION 
 	      && !mutt_strcasecmp(signatures[i]->subtype, "pgp-signature"))
-	    pgp_verify_one (signatures[i], s, tempfile);
+	  {
+	    if (pgp_verify_one (signatures[i], s, tempfile) != 0)
+	      goodsig = 0;
+	  }
 	  else
 	    state_printf (s, _("[-- Warning: We can't verify %s/%s signatures. --]\n\n"),
 			  TYPE(signatures[i]), signatures[i]->subtype);
@@ -640,6 +647,8 @@ void pgp_signed_handler (BODY *a, STATE *s)
       }
       
       mutt_unlink (tempfile);
+
+      b->goodsig = goodsig;
       
       /* Now display the signed body */
       state_puts (_("[-- The following data is signed --]\n\n"), s);
@@ -937,6 +946,16 @@ void pgp_encrypted_handler (BODY *a, STATE *s)
     mutt_body_handler (tattach, s);
     s->fpin = fpin;
 
+    /* 
+     * if a multipart/signed is the _only_ sub-part of a
+     * multipart/encrypted, cache signature verification
+     * status.
+     *
+     */
+    
+    if (mutt_is_multipart_signed (tattach) && !tattach->next)
+      a->goodsig = tattach->goodsig;
+    
     if (s->flags & M_DISPLAY)
       state_puts (_("\n[-- End of PGP/MIME encrypted data --]\n"), s);
 
@@ -1102,15 +1121,17 @@ static BODY *pgp_sign_message (BODY *a)
  */
 char *pgp_findKeys (ADDRESS *to, ADDRESS *cc, ADDRESS *bcc)
 {
-  char *keyID, *keylist = NULL;
+  char *keyID, *keylist = NULL, *t;
   size_t keylist_size = 0;
   size_t keylist_used = 0;
-  ADDRESS *tmp = NULL;
+  ADDRESS *tmp = NULL, *addr = NULL;
   ADDRESS **last = &tmp;
-  ADDRESS *p;
+  ADDRESS *p, *q;
   int i;
   pgp_key_t *k_info, *key;
 
+  const char *fqdn = mutt_fqdn (1);
+  
   for (i = 0; i < 3; i++) 
   {
     switch (i)
@@ -1126,29 +1147,48 @@ char *pgp_findKeys (ADDRESS *to, ADDRESS *cc, ADDRESS *bcc)
       last = &((*last)->next);
   }
 
-  tmp = mutt_remove_duplicates (tmp);
+  if (fqdn)
+    rfc822_qualify (tmp, fqdn);
 
+  tmp = mutt_remove_duplicates (tmp);
+  
   for (p = tmp; p ; p = p->next)
   {
     char buf[LONG_STRING];
 
+    q = p;
     k_info = NULL;
+
     if ((keyID = mutt_pgp_hook (p)) != NULL)
     {
       snprintf (buf, sizeof (buf), _("Use keyID = \"%s\" for %s?"), keyID, p->mailbox);
       if (mutt_yesorno (buf, M_YES) == M_YES)
-	k_info = pgp_getkeybystr (keyID, KEYFLAG_CANENCRYPT, PGP_PUBRING);
+      {
+	/* check for e-mail address */
+	if ((t = strchr (keyID, '@')) && 
+	    (addr = rfc822_parse_adrlist (NULL, keyID)))
+	{
+	  if (fqdn) rfc822_qualify (addr, fqdn);
+	  q = addr;
+	}
+	else
+	  k_info = pgp_getkeybystr (keyID, KEYFLAG_CANENCRYPT, PGP_PUBRING);
+      }
     }
 
-    if (k_info == NULL && (k_info = pgp_getkeybyaddr (p, KEYFLAG_CANENCRYPT, PGP_PUBRING)) == NULL)
+    if (k_info == NULL)
+      pgp_invoke_getkeys (q);
+
+    if (k_info == NULL && (k_info = pgp_getkeybyaddr (q, KEYFLAG_CANENCRYPT, PGP_PUBRING)) == NULL)
     {
-      snprintf (buf, sizeof (buf), _("Enter keyID for %s: "), p->mailbox);
+      snprintf (buf, sizeof (buf), _("Enter keyID for %s: "), q->mailbox);
       
-      if ((key = pgp_ask_for_key (buf, p->mailbox,
+      if ((key = pgp_ask_for_key (buf, q->mailbox,
 				  KEYFLAG_CANENCRYPT, PGP_PUBRING)) == NULL)
       {
 	safe_free ((void **)&keylist);
 	rfc822_free_address (&tmp);
+	rfc822_free_address (&addr);
 	return NULL;
       }
     }
@@ -1164,6 +1204,8 @@ char *pgp_findKeys (ADDRESS *to, ADDRESS *cc, ADDRESS *bcc)
     keylist_used = mutt_strlen (keylist);
 
     pgp_free_key (&key);
+    rfc822_free_address (&addr);
+
   }
   rfc822_free_address (&tmp);
   return (keylist);
@@ -1288,6 +1330,134 @@ static BODY *pgp_encrypt_message (BODY *a, char *keylist, int sign)
   return (t);
 }
 
+static BODY *pgp_traditional_encryptsign (BODY *a, int flags, char *keylist)
+{
+  BODY *b;
+
+  char pgpoutfile[_POSIX_PATH_MAX];
+  char pgperrfile[_POSIX_PATH_MAX];
+  char pgpinfile[_POSIX_PATH_MAX];
+  
+  FILE *pgpout = NULL, *pgperr = NULL, *pgpin = NULL;
+  FILE *fp;
+
+  int empty;
+  int err;
+
+  char buff[STRING];
+
+  pid_t thepid;
+  
+  if ((fp = fopen (a->filename, "r")) == NULL)
+  {
+    mutt_perror (a->filename);
+    return NULL;
+  }
+  
+  mutt_mktemp (pgpinfile);
+  if ((pgpin = safe_fopen (pgpinfile, "w")) == NULL)
+  {
+    mutt_perror (pgpinfile);
+    fclose (fp);
+    return NULL;
+  }
+
+  mutt_copy_stream (fp, pgpin);
+  fclose (fp);
+  fclose (pgpin);
+
+  mutt_mktemp (pgpoutfile);
+  mutt_mktemp (pgperrfile);
+  if ((pgpout = safe_fopen (pgpoutfile, "w+")) == NULL ||
+      (pgperr = safe_fopen (pgperrfile, "w+")) == NULL)
+  {
+    mutt_perror (pgpout ? pgperrfile : pgpoutfile);
+    unlink (pgpinfile);
+    if (pgpout) 
+    {
+      fclose (pgpout);
+      unlink (pgpoutfile);
+    }
+    return NULL;
+  }
+  
+  unlink (pgperrfile);
+  
+  if ((thepid = pgp_invoke_traditional (&pgpin, NULL, NULL, 
+					-1, fileno (pgpout), fileno (pgperr),
+					pgpinfile, keylist, flags)) == -1)
+  {
+    mutt_perror _("Can't invoke PGP");
+    fclose (pgpout);
+    fclose (pgperr);
+    mutt_unlink (pgpinfile);
+    unlink (pgpoutfile);
+    return NULL;
+  }
+
+  if (flags & PGPSIGN)
+    fprintf (pgpin, "%s\n", PgpPass);
+  fclose (pgpin);
+
+  mutt_wait_filter (thepid);
+
+  mutt_unlink (pgpinfile);
+
+  fflush (pgpout);
+  fflush (pgperr);
+
+  rewind (pgpout);
+  rewind (pgperr);
+  
+  empty = (fgetc (pgpout) == EOF);
+  fclose (pgpout);
+  
+  err = 0;
+  
+  while (fgets (buff, sizeof (buff), pgperr))
+  {
+    err = 1;
+    fputs (buff, stdout);
+  }
+  
+  fclose (pgperr);
+  
+  if (err)
+    mutt_any_key_to_continue (NULL);
+  
+  if (empty)
+  {
+    unlink (pgpoutfile);
+    return NULL;
+  }
+    
+  b = mutt_new_body ();
+  
+  b->encoding = ENC7BIT;
+
+  b->type = TYPEAPPLICATION;
+  b->subtype = safe_strdup ("pgp");
+
+  mutt_set_parameter ("format", "text", &b->parameter);
+  mutt_set_parameter ("x-action", flags & PGPENCRYPT ? "encrypt" : "sign",
+		      &b->parameter);
+
+  b->filename = safe_strdup (pgpoutfile);
+  
+  /* The following is intended to give a clue to some completely brain-dead 
+   * "mail environments" which are typically used by large corporations.
+   */
+
+  b->d_filename = safe_strdup ("msg.pgp");
+  b->disposition = DISPINLINE;
+  b->unlink   = 1;
+  b->use_disp = 1;
+
+  return b;
+}
+
+
+
 int pgp_get_keys (HEADER *msg, char **pgpkeylist)
 {
   /* Do a quick check to make sure that we can find all of the encryption
@@ -1311,15 +1481,36 @@ int pgp_protect (HEADER *msg, char *pgpkeylist)
 {
   BODY *pbody = NULL;
   int flags = msg->pgp;
+  int traditional = 0;
+  int i;
 
   if ((msg->pgp & PGPSIGN) && !pgp_valid_passphrase ())
     return (-1);
 
+  if ((msg->content->type == TYPETEXT) &&
+      !mutt_strcasecmp (msg->content->subtype, "plain") &&
+      ((flags & PGPENCRYPT) || (msg->content->content && msg->content->content->hibin == 0)))
+  {
+    if ((i = query_quadoption (OPT_PGPTRADITIONAL, _("Create an application/pgp message?"))) == -1)
+      return -1;
+    else if (i == M_YES)
+      traditional = 1;
+  }
+
   mutt_message _("Invoking PGP...");
-  
+
   if (!isendwin ())
     endwin ();
 
+  if (traditional)
+  {
+    if (!(pbody = pgp_traditional_encryptsign (msg->content, flags, pgpkeylist)))
+      return -1;
+    
+    msg->content = pbody;
+    return 0;
+  }
+
   if ((flags & PGPSIGN) && (!(flags & PGPENCRYPT) || option (OPTPGPRETAINABLESIG)))
   {
     if (!(pbody = pgp_sign_message (msg->content)))
@@ -1346,6 +1537,15 @@ int pgp_protect (HEADER *msg, char *pgpkeylist)
       return (-1);
     }
 
+    /* destroy temporary signature envelope when doing retainable 
+     * signatures.
+     */
+    if (flags != msg->pgp)
+    {
+      mutt_remove_multipart (msg->content);
+      mutt_free_body (&msg->content->next);
+    }
+    
     msg->content = pbody;
   }
 
