@@ -29,6 +29,7 @@ WHERE char *PgpEntryFormat;
 
 /* The command formats */
 
+WHERE char *PgpClearSignCommand;
 WHERE char *PgpDecodeCommand;
 WHERE char *PgpVerifyCommand;
 WHERE char *PgpDecryptCommand;
@@ -40,7 +41,7 @@ WHERE char *PgpExportCommand;
 WHERE char *PgpVerifyKeyCommand;
 WHERE char *PgpListSecringCommand;
 WHERE char *PgpListPubringCommand;
-
+WHERE char *PgpGetkeysCommand;
 
 /* prototypes */
 
@@ -96,7 +97,6 @@ pid_t pgp_invoke_sign (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 pid_t pgp_invoke_encrypt (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			  int pgpinfd, int pgpoutfd, int pgperrfd,
 			  const char *fname, const char *uids, int sign);
-void pgp_invoke_import (const char *fname);
 pid_t pgp_invoke_export (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			 int pgpinfd, int pgpoutfd, int pgperrfd, 
 			 const char *uids);
@@ -106,6 +106,12 @@ pid_t pgp_invoke_verify_key (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 pid_t pgp_invoke_list_keys (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			    int pgpinfd, int pgpoutfd, int pgperrfd, 
 			    pgp_ring_t keyring, LIST *hints);
+pid_t pgp_invoke_traditional (FILE **pgpin, FILE **pgpout, FILE **pgperr,
+			  int pgpinfd, int pgpoutfd, int pgperrfd,
+			  const char *fname, const char *uids, int flags);
 
 
+void pgp_invoke_import (const char *fname);
+void pgp_invoke_getkeys (ADDRESS *);
+
 #endif /* _PGPPATH */
