@@ -23,15 +23,20 @@
  * is almost entirely format based.
  */
 
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <time.h>
 
 #include "mutt.h"
+#include "mutt_curses.h"
 #include "pgp.h"
-
+#include "rfc822.h"
 
 /*
  * The actual command line formatter.
@@ -121,13 +126,18 @@ const char *_mutt_fmt_pgp_command (char *dest,
 	optional = 0;
       break;
     }
+    default:
+    {
+      *dest = '\0';
+      break;
+    }
   }
-  
+
   if (optional)
-    mutt_FormatString (dest, destlen, ifstring, mutt_attach_fmt, data, 0);
+    mutt_FormatString (dest, destlen, ifstring, _mutt_fmt_pgp_command, data, 0);
   else if (flags & M_FORMAT_OPTIONAL)
-    mutt_FormatString (dest, destlen, elsestring, mutt_attach_fmt, data, 0);
-  
+    mutt_FormatString (dest, destlen, elsestring, _mutt_fmt_pgp_command, data, 0);
+
   return (src);
 }
 
@@ -225,6 +235,21 @@ pid_t pgp_invoke_encrypt (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 		       PgpEncryptOnlyCommand);
 }
 
+pid_t pgp_invoke_traditional (FILE **pgpin, FILE **pgpout, FILE **pgperr,
+			      int pgpinfd, int pgpoutfd, int pgperrfd,
+			      const char *fname, const char *uids, int flags)
+{
+  if (flags & PGPENCRYPT)
+    return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
+		       flags & PGPSIGN ? 1 : 0, fname, NULL, PgpSignAs, uids, 
+		       flags & PGPSIGN ? PgpEncryptSignCommand : PgpEncryptOnlyCommand);
+  else
+    return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
+		       1, fname, NULL, PgpSignAs, NULL,
+		       PgpClearSignCommand);
+}
+
+
 void pgp_invoke_import (const char *fname)
 {
   char _fname[_POSIX_PATH_MAX + SHORT_STRING];
@@ -240,6 +265,43 @@ void pgp_invoke_import (const char *fname)
   mutt_system (cmd);
 }
 
+void pgp_invoke_getkeys (ADDRESS *addr)
+{
+  char buff[LONG_STRING];
+  char tmp[LONG_STRING];
+  char cmd[HUGE_STRING];
+  int devnull;
+
+  char *personal;
+  
+  struct pgp_command_context cctx;
+
+  if (!PgpGetkeysCommand) return;
+  
+  memset (&cctx, 0, sizeof (cctx));
+
+  personal = addr->personal;
+  addr->personal = NULL;
+  
+  *tmp = '\0';
+  rfc822_write_address_single (tmp, sizeof (tmp), addr);
+  mutt_quote_filename (buff, sizeof (buff), tmp);
+
+  addr->personal = personal;
+  
+  cctx.ids = buff;
+  
+  mutt_pgp_command (cmd, sizeof (cmd), &cctx, PgpGetkeysCommand);
+
+  devnull = open ("/dev/null", O_RDWR);
+
+  if (!isendwin ()) mutt_message  _("Fetching PGP key...");
+
+  mutt_system (cmd);
+
+  if (!isendwin ()) mutt_clear_error ();
+}
+
 pid_t pgp_invoke_export (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			 int pgpinfd, int pgpoutfd, int pgperrfd, 
 			 const char *uids)
