@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 1996-1998 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999 Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -43,6 +44,15 @@
 #include <string.h>
 #include <errno.h>
 
+static const char Mailbox_is_read_only[] = N_("Mailbox is read-only.");
+
+#define CHECK_READONLY if (Context->readonly) \
+{\
+    mutt_flushinp (); \
+    mutt_error _(Mailbox_is_read_only); \
+    break; \
+}
+
 static struct mapping_t AttachHelp[] = {
   { N_("Exit"),  OP_EXIT },
   { N_("Save"),  OP_SAVE },
@@ -746,7 +756,7 @@ copy_tagged_attachments (FILE *fpout, FILE *fpin, const char *boundary, BODY *bd
   {
     if (bdy->tagged)
     {
-      fprintf (fpout, "--%s\n", boundary);
+      fprintf (fpout, "\n--%s\n", boundary);
       fseek (fpin, bdy->hdr_offset, 0);
       mutt_copy_bytes (fpin, fpout, bdy->length + bdy->offset - bdy->hdr_offset);
     }
@@ -791,6 +801,7 @@ create_tagged_message (const char *tempfile,
     mutt_copy_bytes (src->fp, msg->fp, body->length);
   }
 
+  mx_commit_message (msg, &tmpctx);
   mx_close_message (&msg);
   mx_close_message (&src);
   mx_close_mailbox (&tmpctx);
@@ -803,6 +814,12 @@ create_tagged_message (const char *tempfile,
  * hdr		current message
  * body		current attachment 
  */
+
+/* 
+ * XXX - this code looks way too complicated, and rather ineffective.
+ * Analyze and fix this.
+ */
+
 static void reply_attachment_list (int op, int tag, HEADER *hdr, BODY *body)
 {
   HEADER *hn;
@@ -812,7 +829,11 @@ static void reply_attachment_list (int op, int tag, HEADER *hdr, BODY *body)
   if (!tag && body->hdr)
   {
     hn = body->hdr;
-    hn->msgno = hdr->msgno; /* required for MH/maildir */
+    hn->msgno   = hdr->msgno; /* required for MH/maildir */
+    hn->replied = hdr->replied;
+    hn->read    = hdr->read;
+    hn->old	= hdr->old;
+    hn->changed = hdr->changed;
     ctx = Context;
   }
   else
@@ -839,6 +860,8 @@ static void reply_attachment_list (int op, int tag, HEADER *hdr, BODY *body)
 
     mutt_free_envelope (&newhdr->env);
     newhdr->env = mutt_new_envelope();
+    newhdr->received = 0;
+    newhdr->date_sent = 0;
 
     /* set the default subject for the message. */
     buffer[0] = 0;
@@ -851,8 +874,13 @@ static void reply_attachment_list (int op, int tag, HEADER *hdr, BODY *body)
   else
     ci_send_message (op, NULL, NULL, ctx, hn);
 
-  if (hn->replied && !hdr->replied)
-    mutt_set_flag (Context, hdr, M_REPLIED, 1);
+  if (hn->replied)
+  {
+    if (Context != ctx)
+      mutt_set_flag (Context, hdr, M_REPLIED, 1);
+    else
+      _mutt_set_flag (Context, hdr, M_REPLIED, 1, 0);
+  }
 
 cleanup:
 
@@ -867,9 +895,11 @@ cleanup:
 void
 mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, ATTACHPTR **idx)
 {
+#if 0
   int old_optweed = option (OPTWEED);
-
   set_option (OPTWEED);
+#endif
+  
   do
   {
     switch (op)
@@ -908,8 +938,10 @@ mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, ATTACHPTR **idx)
   }
   while (op != OP_NULL);
 
+#if 0
   if (option (OPTWEED) != old_optweed)
     toggle_option (OPTWEED);
+#endif
 }
 
 
@@ -953,7 +985,7 @@ void mutt_view_attachments (HEADER *hdr)
 #ifdef _PGPPATH
   if((hdr->pgp & PGPENCRYPT) && !pgp_valid_passphrase())
   {
-    mx_close_message(&msg);
+    mx_close_message (&msg);
     return;
   }
   
@@ -1032,6 +1064,9 @@ void mutt_view_attachments (HEADER *hdr)
 	break;
 
       case OP_DELETE:
+	CHECK_READONLY;
+
+
 
 #ifdef _PGPPATH
         if (hdr->pgp)
@@ -1082,6 +1117,7 @@ void mutt_view_attachments (HEADER *hdr)
         break;
 
       case OP_UNDELETE:
+       CHECK_READONLY;
        if (!menu->tagprefix)
        {
 	 idx[menu->current]->content->deleted = 0;
