@@ -19,6 +19,7 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "rfc2047.h"
+#include "rfc2231.h"
 #include "mx.h"
 #include "mime.h"
 #include "mailbox.h"
@@ -348,10 +349,12 @@ int mutt_write_mime_header (BODY *a, FILE *f)
 {
   PARAMETER *p;
   char buffer[STRING];
+  char tmp[STRING];
   char *t;
   char *fn;
   int len;
   int tmplen;
+  int encode;
   
   fprintf (f, "Content-Type: %s/%s", TYPE (a), a->subtype);
 
@@ -368,7 +371,8 @@ int mutt_write_mime_header (BODY *a, FILE *f)
       fputc (';', f);
 
       buffer[0] = 0;
-      rfc822_cat (buffer, sizeof (buffer), p->value, MimeSpecials);
+      encode = rfc2231_encode (tmp, sizeof (tmp), (unsigned char *) p->value);
+      rfc822_cat (buffer, sizeof (buffer), tmp, MimeSpecials);
 
       tmplen = mutt_strlen (buffer) + mutt_strlen (p->attribute) + 1;
 
@@ -383,7 +387,7 @@ int mutt_write_mime_header (BODY *a, FILE *f)
 	len += tmplen + 1;
       }
 
-      fprintf (f, "%s=%s", p->attribute, buffer);
+      fprintf (f, "%s%s=%s", p->attribute, encode ? "*" : "", buffer);
 
     }
   }
@@ -409,8 +413,9 @@ int mutt_write_mime_header (BODY *a, FILE *f)
 	t = fn;
       
       buffer[0] = 0;
-      rfc822_cat (buffer, sizeof (buffer), t, MimeSpecials);
-      fprintf (f, "; filename=%s", buffer);
+      encode = rfc2231_encode (tmp, sizeof (tmp), (unsigned char *) t);
+      rfc822_cat (buffer, sizeof (buffer), tmp, MimeSpecials);
+      fprintf (f, "; filename%s=%s", encode ? "*" : "", buffer);
     }
 
     fputc ('\n', f);
@@ -539,6 +544,7 @@ static CONTENT *mutt_get_content_info (const char *fname, BODY *b)
     }
     else if (ch == '\r')
     {
+      info->cr = 1;
       if ((ch = fgetc (fp)) == EOF)
       {
         info->binary = 1;
@@ -864,7 +870,7 @@ static void mutt_set_encoding (BODY *b, CONTENT *info)
       b->encoding = ENC7BIT;
   }
   else if (info->lobin || info->hibin || info->binary || info->linemax > 990
-	   || (option (OPTENCODEFROM) && info->from))
+	   || info->cr || (option (OPTENCODEFROM) && info->from))
   {
     /* Determine which encoding is smaller  */
     if (1.33 * (float)(info->lobin+info->hibin+info->ascii) < 3.0 * (float) (info->lobin + info->hibin) + (float)info->ascii)
@@ -1805,7 +1811,7 @@ void mutt_prepare_envelope (ENVELOPE *env)
     env->message_id = mutt_gen_msgid ();
 }
   
-void mutt_bounce_message (HEADER *h, ADDRESS *to)
+static void _mutt_bounce_message (HEADER *h, ADDRESS *to, const char *resent_from)
 {
   int i;
   FILE *f;
@@ -1816,7 +1822,7 @@ void mutt_bounce_message (HEADER *h, ADDRESS *to)
   {
     for (i=0; i<Context->msgcount; i++)
       if (Context->hdrs[i]->tagged)
-	mutt_bounce_message (Context->hdrs[i], to);
+	_mutt_bounce_message (Context->hdrs[i], to, resent_from);
     return;
   }
 
@@ -1825,7 +1831,6 @@ void mutt_bounce_message (HEADER *h, ADDRESS *to)
     mutt_mktemp (tempfile);
     if ((f = safe_fopen (tempfile, "w")) != NULL)
     {
-      const char *fqdn;
       int ch_flags = CH_XMIT | CH_NONEWLINE;
       
       if (!option (OPTBOUNCEDELIVERED))
@@ -1833,9 +1838,7 @@ void mutt_bounce_message (HEADER *h, ADDRESS *to)
       
       fseek (msg->fp, h->offset, 0);
       mutt_copy_header (msg->fp, h, f, ch_flags, NULL);
-      fprintf (f, "Resent-From: %s", NONULL(Username));
-      if((fqdn = mutt_fqdn(1)))
-	fprintf (f, "@%s", fqdn);
+      fprintf (f, "Resent-From: %s", resent_from);
       fprintf (f, "\nResent-%s", mutt_make_date (date, sizeof(date)));
       fputs ("Resent-To: ", f);
       mutt_write_address_list (to, f, 11);
@@ -1849,6 +1852,27 @@ void mutt_bounce_message (HEADER *h, ADDRESS *to)
   }
 }
 
+void mutt_bounce_message (HEADER *h, ADDRESS *to)
+{
+  ADDRESS *from;
+  const char *fqdn = mutt_fqdn (1);
+  char resent_from[STRING];
+
+  resent_from[0] = '\0';
+  from = mutt_default_from ();
+
+  if (fqdn)
+    rfc822_qualify (from, fqdn);
+
+  rfc2047_encode_adrlist (from);
+  
+  rfc822_write_address (resent_from, sizeof (resent_from), from);
+  rfc822_free_address (&from);
+  
+  _mutt_bounce_message (h, to, resent_from);
+}
+
+
 /* given a list of addresses, return a list of unique addresses */
 ADDRESS *mutt_remove_duplicates (ADDRESS *addr)
 {
@@ -2008,6 +2032,7 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post,
       dprint (1, (debugfile, "mutt_write_fcc(): %s: write failed.\n", tempfile));
       fclose (tempfp);
       unlink (tempfile);
+      mx_commit_message (msg, &f);	/* XXX - really? */
       mx_close_message (&msg);
       mx_close_mailbox (&f);
       return -1;
@@ -2035,6 +2060,7 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post,
     r = mutt_write_mime_body (hdr->content, msg->fp);
   }
 
+  mx_commit_message (msg, &f);
   mx_close_message (&msg);
   mx_close_mailbox (&f);
 
